Scriptname DD:DD_PlayerRefScript extends ReferenceAlias

;-----------------------------------------------------------------------------------------------
; Devious Devices - DD_PlayerRefScript
; By Kimy
; Additions by EgoBallistic
;-----------------------------------------------------------------------------------------------

DD_Library Property libs Auto Const
; DD_ConfigQuest Property Config Auto Const
GlobalVariable Property DD_GagTalkDoneFlag Auto Const
MiscObject Property DD_TestItem Auto Const
MagicEffect Property PowerArmorPerkEffect Auto Const
Race Property PowerArmorRace Auto

Int GagTalkResetTimer = 1
Int BoundEffectsTimer = 2
Int ArousalUpdateTimer = 3

_TD_GeneralFunctionsScript TD_GeneralFunctions ;EgoBallistic

AAF:AAF_API AAF_API ;EgoBallistic

Event OnInit()
	OnPlayerLoadGame()	
EndEvent

Event OnTimer(Int TimerID)
	If TimerID == GagTalkResetTimer
		DD_GagTalkDoneFlag.SetValue(0)
	EndIf
	If TimerID == BoundEffectsTimer
		libs.ProcessBoundEffects()
	EndIf
	If TimerID == ArousalUpdateTimer
		DisplayArousalMessages()
	EndIf
EndEvent

Event OnPlayerLoadGame()
	AddInventoryEventFilter(None)
	RegisterForMenuOpenCloseEvent("DialogueMenu") 		; We need to register dialogue events for the gag talk
	RegisterForMagicEffectApplyEvent(libs.Player, akCasterFilter = None, akEffectFilter = PowerArmorPerkEffect, abMatch = true) ; We can detect if the player enters any power armor with this event! :D
	RegisterForRemoteEvent(libs.Player, "OnItemUnequipped")
	If libs.GetVersion() > libs.CurrentVersion
		libs.updateMod()
		libs.notify("Devious Devices Version " + libs.GetVersionString() + " initialized!")
		libs.CurrentVersion = libs.GetVersion()
	EndIf
	libs.ProcessBoundEffects()
	;libs.PopulateDB() //deprecated
	
	StartTimer(900, ArousalUpdateTimer) ;15 minutes
	
	LoadAAF() ;EgoBallistic
	TD_GeneralFunctions = Game.GetFormFromFile(0x00173A, "TortureDevices.esm") as _TD_GeneralFunctionsScript
	if TD_GeneralFunctions.GetVersion() < 2.2
		Debug.Messagebox("Devious Devices Error: Torture Devices version 2.2 or higher is required.")
	endif
	
	libs.LoadSexAttributessArousal()
EndEvent

Function LoadAAF() ;EgoBallistic
	AAF_API = Game.GetFormFromFile(0x00000F99, "AAF.esm") as AAF:AAF_API
	If !AAF_API
		libs.notify("Devious Devices: AAF API not found.")
		libs.AAF_ActorBusy = none
		Libs.Config.HasAAF = false
		Libs.Config.NoAAF = true
	Else
		RegisterForCustomEvent(AAF_API, "OnSceneInit")
		RegisterForCustomEvent(AAF_API, "OnAnimationStart")
        RegisterForCustomEvent(AAF_API, "OnAnimationStop")
		libs.AAF_ActorBusy = AAF_API.AAF_ActorBusy
		Libs.Config.HasAAF = true
		Libs.Config.NoAAF = false
	EndIf
EndFunction

Bool AAFBlindfoldRemoved = false

Bool Function DetectPlayer(Actor[] actors)
    
    Int J = 0
    While (J < actors.length)
        If (Actors[J] == libs.player)
            return true
        EndIf
       J += 1
    EndWhile
    return false
EndFunction

Event OnMagicEffectApply(ObjectReference akTarget, ObjectReference akCaster, MagicEffect akEffect)
	; sanity check, but the filter should route only this event our way anyway!
	If akTarget != Libs.Player || akEffect != PowerArmorPerkEffect
		return
	EndIf
	If Libs.Player.HasKeyword(libs.DD_kw_EffectType_BlockPowerArmor)
		; bump her out
		Libs.ExitPowerArmor(Libs.Player)
		Utility.Wait(1)
		Libs.Notify("You cannot operate power armor while restrained!")
	EndIf
	; apparently one needs to re-register this event, otherwise it will trigger only once. Silly Bethesda!
	RegisterForMagicEffectApplyEvent(libs.Player, akCasterFilter = None, akEffectFilter = PowerArmorPerkEffect, abMatch = true)
EndEvent

Event OnRaceSwitchComplete()
	; this script will be called when the player exits a power armor, as the power armor involves a race change.
	If libs.player.GetRace() != PowerArmorRace && libs.player.WornHasKeyword(libs.DD_kw_RenderedItem)
		; we will need to reequip all rendered items, as a silly bug in the game engine wipes the "don't unequip this" flag from the Rendered items
		ReEquipRenderedDevices()
	EndIf
endEvent

Function ReEquipRenderedDevices()
	Armor iDevice
	Armor rDevice
	Int i = libs.DD_PlayerEquippedItems.GetSize()
	While i > 0
		i -= 1
		iDevice = libs.DD_PlayerEquippedItems.GetAt(i) As Armor
		rDevice = libs.GetRenderedDevice(iDevice)
		libs.Player.RemoveItem(rDevice, 1, True)
		libs.Player.EquipItem(rDevice, True, True)
	EndWhile
EndFunction

Event OnItemUnequipped(Form akBaseObject, ObjectReference akReference)
	If !akBaseObject.HasKeyword(libs.DD_kw_RenderedItem)
		return
	EndIf
	libs.log("Script: DD_PlayerRefScript. Event: OnItemEquipped - Actor: " + libs.player.GetActorBase().getname() + ". RenderedDevice: " + akBaseObject + ". Reason: Trigger event!")
	Armor iDevice
	Armor rDevice
	Int i = libs.DD_PlayerEquippedItems.GetSize()
	While i > 0
		i -= 1
		iDevice = libs.DD_PlayerEquippedItems.GetAt(i) As Armor
		rDevice = libs.GetRenderedDevice(iDevice)
		If rDevice == akBaseObject
			If libs.player.IsEquipped(iDevice)
				libs.log("Script: DD_PlayerRefScript. Event: OnItemEquipped - Actor: " + libs.player.GetActorBase().getname() + ". Armor: " + iDevice.getname() + ". Reason: Illegal rendered device unequip!")
				libs.player.Unequipitem(iDevice, false, true)
			EndIf	
			i = -1
		EndIf
	EndWhile
EndEvent

Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
	; This is an easy method to test scripting routines in game:
	If akBaseItem == DD_TestItem
		;ReEquipRenderedDevices()
	EndIf
	; A new restraint was locked on the player, so we process the bound effects
	If akBaseItem.HasKeyword(libs.DD_kw_RenderedItem)
		StartTimer(0.5, BoundEffectsTimer)
	EndIf
EndEvent

Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
	; A restraint was removed from the player, so we process the bound effects.
	If akBaseItem.HasKeyword(libs.DD_kw_RenderedItem)
		StartTimer(0.5, BoundEffectsTimer)
	EndIf
EndEvent

Event OnMenuOpenCloseEvent(string asMenuName, bool abOpening)
    if (asMenuName== "DialogueMenu")
        if !abOpening
			StartTimer(10, GagTalkResetTimer) ; give her a few seconds to initiate the next dialogue.
        endif
    endif
endEvent

; This will apply all the DD and TD animation and animArchetype keywords that are
; on the player to any actor.  The real purpose of this is to ensure that the AAF
; Doppelganger has the same animations as the player when the player has DD equipped
; by EgoBallistic
Function SynchAnimsWithPlayer(Actor akActor)
	Debug.Trace("DD: Synch player animations with " + akActor)

	; check if there is at least one bound keyword on the player, apply same to akActor
	Bool PlayerIsBound = False
	Int i = Libs.DD_BoundAnimKeywords.Length
	While i > 0
		i -= 1
		If Libs.Player.HasKeyword(Libs.DD_BoundAnimKeywords[i])
			akActor.AddKeyword(Libs.DD_BoundAnimKeywords[i])
			PlayerIsBound = True
		EndIf
	EndWhile
	
	If PlayerIsBound
		; We know the player is bound, so apply the specific TD animation keywords to akActor
		Debug.Trace("DD: Player is bound, apply bound keywords to " + akActor)
		i = TD_GeneralFunctions._TD_AnimKeys.Length
		While i > 0
			i -= 1
			If Libs.Player.HasKeyword(TD_GeneralFunctions._TD_AnimKeys[i])
				akActor.AddKeyword(TD_GeneralFunctions._TD_AnimKeys[i])
			EndIf
		EndWhile
		
		; Now apply swap the archetype to kick off the animations
		akActor.ChangeAnimArchetype()
		akActor.ChangeAnimArchetype(Libs._TD_AnimArchetypeBound)
	EndIf
EndFunction

Function DisplayArousalMessages()
	if Libs.config.ShowDDArousalMessages == 2
		libs.GetArousalFeedback(libs.player)
	endif
	StartTimer(900, ArousalUpdateTimer) ;15 minutes
EndFunction

; Detect when the player starts an animation and synch up the AAF Doppelganger's DD animations
; by EgoBallistic
Event AAF:AAF_API.OnSceneInit(AAF:AAF_API akSender, Var[] akArgs)
	Debug.Trace("DD: AAF OnSceneInit")
	Int iResult =  akArgs[0] as Int
	If iResult == 0
		Actor Doppelganger = akArgs[2] as Actor
		If (Doppelganger)
			Debug.Trace("DD: Player in AAF animation")
			SynchAnimsWithPlayer(Doppelganger)
		EndIf
	EndIf
EndEvent

;  Blindfold Effect removing events.

Event AAF:AAF_API.OnAnimationStart(AAF:AAF_API akSender, Var[] akArgs)
	If Libs.Config.ConfBlindfolEffect
        Actor[] actors = Utility.VarToVarArray(akArgs[1]) as Actor[]
        If DetectPlayer(actors) && libs.Player.WornHasKeyword(libs.DD_kw_EffectType_Blindfold)            ; player detected and wear a blindfold
            libs.DD_BlindfoldImageSpace.Remove()            ; taking off blindfold effect
            AAFBlindfoldRemoved = true                      ; mark that we removed a effect
        EndIf
    EndIf    
EndEvent
    
Event AAF:AAF_API.OnAnimationStop(AAF:AAF_API akSender, Var[] akArgs)
    If Libs.Config.ConfBlindfolEffect || AAFBlindfoldRemoved
        Actor[] actors = Utility.VarToVarArray(akArgs[1]) as Actor[]
        If DetectPlayer(actors) && libs.Player.WornHasKeyword(libs.DD_kw_EffectType_Blindfold)  && AAFBlindfoldRemoved    ; player detected and wear a blindfold and effect was removed
            libs.DD_BlindfoldImageSpace.Remove()
            libs.DD_BlindfoldImageSpace.Apply(libs.BlindfoldStrength)
            AAFBlindfoldRemoved = false            ; we restore effect so reset variable
        
            ; will not work             libs.ApplyBlindfold()    ; restore blindfold effect - it can use even if player not wear a blinfold as this fuction runs any time after apply new restraint
        EndIf  
	EndIf
EndEvent
